'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _require = require('express'),
    Router = _require.Router;

var _require2 = require('lodash'),
    camelCase = _require2.camelCase,
    lowerCase = _require2.lowerCase;

var _require3 = require('pluralize'),
    plural = _require3.plural,
    singular = _require3.singular;

var Route = require('./route');
var errors = require('./errors');

var _require4 = require('./utils/helpers'),
    expect = _require4.expect,
    middlify = _require4.middlify,
    hookify = _require4.hookify,
    permissionify = _require4.permissionify,
    orderRoutes = _require4.orderRoutes;

var _require5 = require('./utils/defaults'),
    find = _require5.find,
    count = _require5.count,
    findOne = _require5.findOne,
    findById = _require5.findById,
    create = _require5.create,
    update = _require5.update,
    remove = _require5.remove;

module.exports = function () {

  /**
   * Routes are used as endpoints for an API.
   *
   * @param {string} name the name of the resource used for making paths and variables.
   * @param {string} model the model as created by hobson.
   */
  function Resource() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        model = _ref.model,
        name = _ref.name,
        address = _ref.address,
        _ref$options = _ref.options,
        options = _ref$options === undefined ? {} : _ref$options,
        _ref$unsecure = _ref.unsecure,
        unsecure = _ref$unsecure === undefined ? false : _ref$unsecure;

    _classCallCheck(this, Resource);

    expect({ name: 'model', value: model, type: 'function' });
    expect({ name: 'name', value: name, type: 'string', optional: true });
    expect({ name: 'address', value: name, type: 'string', optional: true });
    expect({ name: 'unsecure', value: unsecure, type: 'boolean', optional: true });
    expect({ name: 'options', value: options, type: 'object' });
    var nickname = name || model.modelName;
    this.name = camelCase(singular(nickname));
    this.address = address || '/' + camelCase(plural(nickname));
    this.model = model;
    this.options = Object.assign({ name: this.name }, this.model.schema.context.options, options);
    this.routes = new Map();
    this.unsecure = unsecure;
    this.defaults();
  }

  /**
   * Set the defaults onto the resource.
   */


  _createClass(Resource, [{
    key: 'defaults',
    value: function defaults() {
      var _this = this;

      this.add({
        id: 'find',
        path: '/',
        method: 'get',
        handler: function handler() {
          return find(_this.options).apply(undefined, arguments);
        }
      });
      this.add({
        id: 'count',
        path: '/count',
        method: 'get',
        handler: function handler() {
          return count(_this.options).apply(undefined, arguments);
        }
      });
      this.add({
        id: 'findOne',
        path: '/one',
        method: 'get',
        handler: function handler() {
          return findOne(_this.options).apply(undefined, arguments);
        }
      });
      this.add({
        id: 'findById',
        path: '/:' + this.name + 'Id',
        method: 'get',
        handler: function handler() {
          return findById(_this.options).apply(undefined, arguments);
        }
      });
      this.add({
        id: 'create',
        path: '/',
        method: 'post',
        handler: function handler() {
          return create(_this.options).apply(undefined, arguments);
        }
      });
      this.add({
        id: 'update',
        path: '/:' + this.name + 'Id',
        method: 'patch',
        handler: function handler() {
          return update(_this.options).apply(undefined, arguments);
        }
      });
      this.add({
        id: 'remove',
        path: '/:' + this.name + 'Id',
        method: 'delete',
        handler: function handler() {
          return remove(_this.options).apply(undefined, arguments);
        }
      });
    }

    /**
     * Set some options on a resource.
     *
     * @param {object} option an object with key/value pairs to set as options.
     */

  }, {
    key: 'option',
    value: function option() {
      var _option = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      this.options = Object.assign({}, _option, this.options);
    }

    /**
     * Add a route to the resource.
     *
     * @param {object} data the route to add to the resource.
     */

  }, {
    key: 'add',
    value: function add(data) {
      expect({ name: 'data', value: data, type: 'object' });
      if (typeof data.access === 'function' && typeof data.before === 'function' && typeof data.after === 'function') {
        this.routes.set(data.id, data);
      } else {
        this.routes.set(data.id, new Route(data));
      }
      this.last = data.id;
      return this;
    }

    /**
     * Get the route with the id provided.
     *
     * @param {string} id the id matching the route.
     */

  }, {
    key: 'get',
    value: function get(id) {
      expect({ name: 'id', value: id, type: 'string' });
      if (this.routes.has(id)) {
        return this.routes.get(id);
      }
      throw new errors.BreakingResponse({ message: 'No route with the id "' + id + '" was found on the resource.' });
    }

    /**
     * Get the last route which was added to the resource.
     */

  }, {
    key: 'refine',
    value: function refine() {
      if (!this.last) {
        throw new errors.BreakingResponse({ message: 'Route muse be added before calling refine().' });
      }
      if (!this.routes.has(this.last)) {
        throw new errors.BreakingResponse({ message: 'Last route no longer exists on the resource.' });
      }
      return this.routes.get(this.last);
    }

    /**
     * Generate a express router from the resource.
     */

  }, {
    key: 'compile',
    value: function compile() {
      var _this2 = this;

      var router = Router();
      [].concat(_toConsumableArray(this.routes.entries())).sort(orderRoutes).forEach(function (_ref2) {
        var _ref3 = _slicedToArray(_ref2, 2),
            key = _ref3[0],
            route = _ref3[1];

        var unsecure = typeof route.open === 'boolean' ? route.open : _this2.unsecure;
        var resources = {
          Model: _this2.model,
          context: {} // empty object which can be used to pass information between methods
        };
        var permission = middlify(permissionify(key, route.permissions, unsecure), resources);
        var hooked = hookify(key, route.handler, route.befores, route.afters);
        var work = middlify(hooked, resources, true);
        router[lowerCase(route.method)].apply(router, [route.path].concat(_toConsumableArray(route.middlewares), [permission, work]));
      });
      return router;
    }

    /**
     * Attach the resources routes to an express app.
     *
     * @param {function} app the express app instance.
     */

  }, {
    key: 'attach',
    value: function attach(app) {
      if (app && typeof app.use === 'function') {
        return app.use(this.address, this.compile());
      }
      throw new errors.BreakingResponse({ message: 'Expected "app" paramater to be an express app but got ' + (typeof app === 'undefined' ? 'undefined' : _typeof(app)) + '.' });
    }
  }]);

  return Resource;
}();