'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _require = require('./utils/helpers'),
    expect = _require.expect;

module.exports = function () {

  /**
   * Routes are used as endpoints for an API.
   *
   * @param {string} id the id of the route used in referencing it.
   * @param {string} path the (sub) path of the route.
   * @param {string} method the method (e.g. get, post, put) of the route.
   * @param {function} handler the main function of the route.
   * @param {boolean} open set this route as unprotected by default (not recommended).
   */
  function Route() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        id = _ref.id,
        path = _ref.path,
        method = _ref.method,
        handler = _ref.handler,
        _ref$open = _ref.open,
        open = _ref$open === undefined ? false : _ref$open;

    _classCallCheck(this, Route);

    expect({ name: 'id', value: id, type: 'string' });
    expect({ name: 'path', value: path, type: 'string' });
    expect({ name: 'method', value: method, type: 'string' });
    expect({ name: 'handler', value: handler, type: 'function' });
    expect({ name: 'open', value: open, type: 'boolean' });
    this.id = id;
    this.path = path;
    this.method = method;
    this.handler = handler;
    this.open = open;
    this.middlewares = [];
    this.permissions = [];
    this.befores = [];
    this.afters = [];
  }

  /**
   * Add a middleware to the route.
   *
   * @param {function} middleware an express middleware function.
   */


  _createClass(Route, [{
    key: 'middleware',
    value: function middleware(_middleware) {
      expect({ name: 'middleware', value: _middleware, type: 'function' });
      this.middlewares.push(_middleware);
      return this;
    }

    /**
     * Add a permission to the route.
     *
     * @param {function} permission a function (or async function) which resolved to true if allowed.
     */

  }, {
    key: 'access',
    value: function access(permission) {
      expect({ name: 'permission', value: permission, type: 'function' });
      this.permissions.push(permission);
      return this;
    }

    /**
     * Add a before hook to the route.
     *
     * @param {function} before a function (or async function) which is run before the route handler.
     */

  }, {
    key: 'before',
    value: function before(_before) {
      expect({ name: 'before', value: _before, type: 'function' });
      this.befores.push(_before);
      return this;
    }

    /**
     * Add a after hook to the route.
     *
     * @param {function} after a function (or async function) which is run after the route handler.
     */

  }, {
    key: 'after',
    value: function after(_after) {
      expect({ name: 'after', value: _after, type: 'function' });
      this.afters.push(_after);
      return this;
    }
  }]);

  return Route;
}();