'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var mongoose = require('mongoose');

var _require = require('./utils/helpers'),
    expect = _require.expect;

module.exports = function (_mongoose$Schema) {
  _inherits(Schema, _mongoose$Schema);

  /**
   * Routes are used as endpoints for an API.
   *
   * @param {string} id the id of the route used in referencing it.
   * @param {string} path the (sub) path of the route.
   * @param {string} method the method (e.g. get, post, put) of the route.
   * @param {function} handler the main function of the route.
   * @param {boolean} open set this route as unprotected by default (not recommended).
   */
  function Schema() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        type = _ref.type,
        _ref$shape = _ref.shape,
        shape = _ref$shape === undefined ? {} : _ref$shape,
        _ref$options = _ref.options,
        options = _ref$options === undefined ? {} : _ref$options,
        _ref$timestamps = _ref.timestamps,
        timestamps = _ref$timestamps === undefined ? true : _ref$timestamps,
        _ref$safe = _ref.safe,
        safe = _ref$safe === undefined ? true : _ref$safe;

    _classCallCheck(this, Schema);

    expect({ name: 'type', value: type, type: 'string', optional: true });
    expect({ name: 'shape', value: shape, type: 'object' });
    expect({ name: 'options', value: options, type: 'object' });
    expect({ name: 'timestamps', value: timestamps, type: 'boolean' });
    expect({ name: 'safe', value: safe, type: 'boolean' });
    var configuration = Object.assign({ timestamps: timestamps, safe: safe }, options);
    var mixins = {};
    if (safe) {
      Object.assign(mixins, {
        deleted: {
          type: Boolean,
          required: true,
          default: false
        }
      });
      if (configuration.timestamps) {
        Object.assign(mixins, {
          deletedAt: {
            type: Date
          }
        });
      }
    }
    var structure = Object.assign(mixins, shape);

    var _this = _possibleConstructorReturn(this, (Schema.__proto__ || Object.getPrototypeOf(Schema)).call(this, _extends({}, structure), _extends({}, configuration)));

    _this.context = {
      options: configuration,
      shape: structure
    };
    return _this;
  }

  /**
   * Defines a model on the database with this schema.
   *
   * @param {string} name the name of the model to be saved to the database.
   */


  _createClass(Schema, [{
    key: 'compile',
    value: function compile(name) {
      expect({ name: 'name', value: name, type: 'string' });
      return mongoose.model(name, this);
    }
  }]);

  return Schema;
}(mongoose.Schema);