'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('../utils/helpers'),
    expect = _require.expect;

var errors = require('../errors');

/**
 * Login a user with the provided credentials.
 */
module.exports.login = function login() {
  var _this = this;

  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      secret = _ref.secret;

  expect({ name: 'secret', value: secret, type: 'string' });
  return function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(_ref2) {
      var Model = _ref2.Model,
          _ref2$body = _ref2.body,
          email = _ref2$body.email,
          password = _ref2$body.password;

      var user, match, auth, _auth$payload, payload;

      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return Model.findOne({ email: { $regex: new RegExp(email, 'i') } }).select('password');

            case 2:
              user = _context.sent;

              if (user) {
                _context.next = 5;
                break;
              }

              throw new errors.NotFoundResponse({ message: 'No user was found for the given email.' });

            case 5:
              _context.next = 7;
              return user.comparePassword(password);

            case 7:
              match = _context.sent;

              if (match) {
                _context.next = 10;
                break;
              }

              throw new errors.BadResponse({ message: 'Password is incorrect.' });

            case 10:
              _context.next = 12;
              return user.tokenize({ secret: secret });

            case 12:
              auth = _context.sent;
              _auth$payload = auth.payload, payload = _auth$payload === undefined ? {} : _auth$payload;
              return _context.abrupt('return', {
                auth: _extends({
                  token: auth.token
                }, payload)
              });

            case 15:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    }));

    return function (_x2) {
      return _ref3.apply(this, arguments);
    };
  }();
};

/**
 * Register (sign up) a new user.
 */
module.exports.register = function register() {
  var _this2 = this;

  var _ref4 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      secret = _ref4.secret;

  expect({ name: 'secret', value: secret, type: 'string' });
  return function () {
    var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(_ref5) {
      var Model = _ref5.Model,
          body = _ref5.body;

      var existing, user, auth, _auth$payload2, payload;

      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return Model.findOne({ email: { $regex: new RegExp(body.email, 'i') } });

            case 2:
              existing = _context2.sent;

              if (!existing) {
                _context2.next = 5;
                break;
              }

              throw new errors.BadResponse({ message: 'User already exists with this email.' });

            case 5:
              _context2.next = 7;
              return Model.create(body);

            case 7:
              user = _context2.sent;

              if (user) {
                _context2.next = 10;
                break;
              }

              throw new errors.BreakingResponse({ message: 'Error occurred while creating user.' });

            case 10:
              _context2.next = 12;
              return user.tokenize({ secret: secret });

            case 12:
              auth = _context2.sent;
              _auth$payload2 = auth.payload, payload = _auth$payload2 === undefined ? {} : _auth$payload2;
              return _context2.abrupt('return', {
                user: user,
                auth: _extends({
                  token: auth.token
                }, payload)
              });

            case 15:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this2);
    }));

    return function (_x4) {
      return _ref6.apply(this, arguments);
    };
  }();
};

/**
 * Logout a user from the application.
 */
module.exports.logout = function logout() {
  var _this3 = this;

  return function () {
    var _ref8 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(_ref7) {
      var auth = _ref7.auth;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              if (!auth) {
                _context3.next = 3;
                break;
              }

              _context3.next = 3;
              return Object.assign(auth, { active: false }).save();

            case 3:
              return _context3.abrupt('return', { auth: null });

            case 4:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, _this3);
    }));

    return function (_x5) {
      return _ref8.apply(this, arguments);
    };
  }();
};

/**
 * Change a user's password.
 */
module.exports.changePassword = function changePassword() {
  var _this4 = this;

  return function () {
    var _ref10 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(_ref9) {
      var Model = _ref9.Model,
          _ref9$body = _ref9.body,
          oldPassword = _ref9$body.oldPassword,
          newPassword = _ref9$body.newPassword,
          user = _ref9.user;
      var patch, match;
      return regeneratorRuntime.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              if (newPassword) {
                _context4.next = 2;
                break;
              }

              throw new errors.BadResponse({
                message: 'There was an error changing password.',
                data: {
                  newPassword: { message: 'The new password field is required.' }
                }
              });

            case 2:
              _context4.next = 4;
              return Model.findById(user.id).select('password');

            case 4:
              patch = _context4.sent;

              if (!(patch && patch.password)) {
                _context4.next = 13;
                break;
              }

              if (oldPassword) {
                _context4.next = 8;
                break;
              }

              throw new errors.BadResponse({
                message: 'There was an error changing password.',
                data: {
                  oldPassword: { message: 'The old password field is required.' }
                }
              });

            case 8:
              _context4.next = 10;
              return patch.comparePassword(oldPassword);

            case 10:
              match = _context4.sent;

              if (match) {
                _context4.next = 13;
                break;
              }

              throw new errors.BadResponse({ message: 'Current password is incorrect.' });

            case 13:
              _context4.next = 15;
              return Object.assign(patch, { password: newPassword }).save();

            case 15:
              delete patch.password;
              return _context4.abrupt('return', { user: patch });

            case 17:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, _this4);
    }));

    return function (_x6) {
      return _ref10.apply(this, arguments);
    };
  }();
};

/**
 * Request that a user's password needs to be updated.
 */
module.exports.forgotPassword = function forgotPassword() {
  var _this5 = this;

  var _ref11 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      secret = _ref11.secret;

  expect({ name: 'secret', value: secret, type: 'string' });
  return function () {
    var _ref13 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(_ref12) {
      var Model = _ref12.Model,
          email = _ref12.body.email,
          context = _ref12.context;
      var user, auth;
      return regeneratorRuntime.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              if (email) {
                _context5.next = 2;
                break;
              }

              throw new errors.BadResponse({
                message: 'There was an error requesting a new password.',
                data: {
                  email: { message: 'The email field is required.' }
                }
              });

            case 2:
              _context5.next = 4;
              return Model.findOne({ email: { $regex: new RegExp(email, 'i') } });

            case 4:
              user = _context5.sent;

              if (user) {
                _context5.next = 7;
                break;
              }

              throw new errors.BadResponse({ message: 'No accounts match the email address: ' + email + '.' });

            case 7:
              _context5.next = 9;
              return user.tokenize({ secret: secret, options: { expiresIn: '1h' } });

            case 9:
              auth = _context5.sent;

              Object.assign(context, { auth: auth });
              return _context5.abrupt('return', {});

            case 12:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, _this5);
    }));

    return function (_x8) {
      return _ref13.apply(this, arguments);
    };
  }();
};

/**
 * Change a user's password.
 */
module.exports.resetPassword = function resetPassword() {
  var _this6 = this;

  return function () {
    var _ref15 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(_ref14) {
      var Model = _ref14.Model,
          auth = _ref14.auth,
          _ref14$body = _ref14.body,
          email = _ref14$body.email,
          newPassword = _ref14$body.newPassword;
      var data, user;
      return regeneratorRuntime.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              if (!(!email || !newPassword)) {
                _context6.next = 5;
                break;
              }

              data = {};

              if (!email) data.email = { message: 'The email field is required.' };
              if (!newPassword) data.newPassword = { message: 'The new password field is required.' };
              throw new errors.BadResponse({ message: 'There was an error resetting password.', data: data });

            case 5:
              _context6.next = 7;
              return Model.findOne({ email: { $regex: new RegExp(email, 'i') } }).select('password');

            case 7:
              user = _context6.sent;

              if (user) {
                _context6.next = 10;
                break;
              }

              throw new errors.BadResponse({ message: 'No accounts match the email address: ' + email + '.' });

            case 10:
              if (!(auth.payload.userId !== user.id)) {
                _context6.next = 12;
                break;
              }

              throw new errors.BadResponse({ message: 'The token does not match the email provided: ' + email + '.' });

            case 12:
              _context6.next = 14;
              return Object.assign(user, { password: newPassword }).save();

            case 14:
              return _context6.abrupt('return', {});

            case 15:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, _this6);
    }));

    return function (_x9) {
      return _ref15.apply(this, arguments);
    };
  }();
};