'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var bcrypt = require('bcryptjs');
var Schema = require('../schema');

var _require = require('../utils/helpers'),
    emailRegex = _require.emailRegex;

var _require2 = require('../utils/auth'),
    authPackage = _require2.authPackage;

var errors = require('../errors');
var Token = require('../token/model');

module.exports = function (_Schema) {
  _inherits(UserSchema, _Schema);

  function UserSchema() {
    var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, UserSchema);

    var _this = _possibleConstructorReturn(this, (UserSchema.__proto__ || Object.getPrototypeOf(UserSchema)).call(this, args));

    _this.add({
      email: {
        type: String,
        required: true,
        unique: true,
        trim: true,
        validate: {
          validator: function validator(email) {
            return emailRegex.test(email);
          },
          message: 'not a valid email format e.g. example@email.com'
        }
      },
      password: {
        type: String,
        select: false,
        required: !args.optionalPassword
      }
    });
    _this.pre('save', function preSave(next) {
      var _this2 = this;

      if (!this.isModified('password')) {
        next();
      } else {
        bcrypt.genSalt(5).then(function (salt) {
          return bcrypt.hash(_this2.password, salt);
        }).then(function (hash) {
          _this2.password = hash;
          next();
        }).catch(next);
      }
    });
    _this.methods.comparePassword = function comparePassword(candidate) {
      if (!this.password) {
        throw new errors.BreakingResponse({ message: 'User has not been configured with a password.' });
      }
      if (!candidate) {
        return false;
      }
      return bcrypt.compare(candidate, this.password);
    };
    _this.methods.tokenize = function tokenize() {
      var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          secret = _ref.secret,
          options = _ref.options;

      var item = new Token({});
      var pack = authPackage({
        id: item.id,
        userId: this.id,
        email: this.email
      }, secret, options);
      return Object.assign(item, pack).save();
    };
    return _this;
  }

  return UserSchema;
}(Schema);