'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('pluralize'),
    plural = _require.plural,
    singular = _require.singular;

var _require2 = require('./helpers'),
    expect = _require2.expect,
    checkObjectId = _require2.checkObjectId;

var _require3 = require('lodash'),
    merge = _require3.merge;

var errors = require('../errors');

/**
 * Find many items in the database.
 *
 * @param {string} name the resource name
 */
module.exports.find = function find() {
  var _this = this;

  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref.name,
      safe = _ref.safe;

  expect({ name: 'name', value: name, type: 'string' });
  return function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(_ref2) {
      var _ref2$query = _ref2.query,
          filter = _ref2$query.filter,
          skip = _ref2$query.skip,
          limit = _ref2$query.limit,
          include = _ref2$query.include,
          sort = _ref2$query.sort,
          select = _ref2$query.select,
          Model = _ref2.Model;
      var options, query, value;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              options = {};

              if (safe) options = { deleted: false };
              query = Model.find(merge(options, filter || {}));

              if (include) query.populate(include);
              if (sort) query.sort(sort);
              if (select) query.select(select);
              if (skip) query.skip(Number(skip));
              if (limit) query.limit(Number(limit));
              _context.next = 10;
              return query.exec();

            case 10:
              value = _context.sent;

              if (value) {
                _context.next = 13;
                break;
              }

              throw new errors.BreakingResponse({ message: 'Error occurred when attempting to query the "' + name + '" model.' });

            case 13:
              return _context.abrupt('return', _defineProperty({}, plural(name), value));

            case 14:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    }));

    return function (_x2) {
      return _ref3.apply(this, arguments);
    };
  }();
};

/**
 * Count the number of items in the database.
 *
 * @param {string} name the resource name
 */
module.exports.count = function count() {
  var _this2 = this;

  var _ref5 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref5.name,
      safe = _ref5.safe;

  expect({ name: 'name', value: name, type: 'string' });
  return function () {
    var _ref7 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(_ref6) {
      var filter = _ref6.query.filter,
          Model = _ref6.Model;
      var options, value;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              options = {};

              if (safe) options = { deleted: false };
              _context2.next = 4;
              return Model.count(merge(options, filter || {}));

            case 4:
              value = _context2.sent;

              if (!(typeof value !== 'number')) {
                _context2.next = 7;
                break;
              }

              throw new errors.BreakingResponse({ message: 'Error occurred when attempting to query the "' + name + '" model.' });

            case 7:
              return _context2.abrupt('return', {
                count: value
              });

            case 8:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this2);
    }));

    return function (_x4) {
      return _ref7.apply(this, arguments);
    };
  }();
};

/**
 * Find one item in the database.
 *
 * @param {string} name the resource name
 */
module.exports.findOne = function findOne() {
  var _this3 = this;

  var _ref8 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref8.name,
      safe = _ref8.safe;

  expect({ name: 'name', value: name, type: 'string' });
  return function () {
    var _ref10 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(_ref9) {
      var Model = _ref9.Model,
          _ref9$query = _ref9.query,
          filter = _ref9$query.filter,
          include = _ref9$query.include,
          select = _ref9$query.select;
      var options, query, value;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              options = {};

              if (safe) options = { deleted: false };
              query = Model.findOne(merge(options, filter || {}));

              if (include) query.populate(include);
              if (select) query.select(select);
              _context3.next = 7;
              return query.exec();

            case 7:
              value = _context3.sent;

              if (value) {
                _context3.next = 10;
                break;
              }

              throw new errors.NotFoundResponse({ message: 'Model ' + name + ' did not have an item with the given parameters.' });

            case 10:
              return _context3.abrupt('return', _defineProperty({}, singular(name), value));

            case 11:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, _this3);
    }));

    return function (_x6) {
      return _ref10.apply(this, arguments);
    };
  }();
};

/**
 * Find one item in the database by it's id.
 *
 * @param {string} name the resource name
 */
module.exports.findById = function findById() {
  var _this4 = this;

  var _ref12 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref12.name;

  expect({ name: 'name', value: name, type: 'string' });
  return function () {
    var _ref14 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(_ref13) {
      var params = _ref13.params,
          Model = _ref13.Model,
          _ref13$query = _ref13.query,
          include = _ref13$query.include,
          select = _ref13$query.select;
      var id, query, value;
      return regeneratorRuntime.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              id = params[name + 'Id'];

              checkObjectId(id);
              query = Model.findById(id);

              if (include) query.populate(include);
              if (select) query.select(select);
              _context4.next = 7;
              return query.exec();

            case 7:
              value = _context4.sent;

              if (value) {
                _context4.next = 10;
                break;
              }

              throw new errors.NotFoundResponse({ message: 'Model ' + name + ' did not have an item with the id "' + id + '".' });

            case 10:
              return _context4.abrupt('return', _defineProperty({}, singular(name), value));

            case 11:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, _this4);
    }));

    return function (_x8) {
      return _ref14.apply(this, arguments);
    };
  }();
};

/**
 * Create a resource item in the database.
 *
 * @param {string} name the resource name
 */
module.exports.create = function create() {
  var _this5 = this;

  var _ref16 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref16.name;

  expect({ name: 'name', value: name, type: 'string' });
  return function () {
    var _ref18 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(_ref17) {
      var body = _ref17.body,
          Model = _ref17.Model;
      var value;
      return regeneratorRuntime.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return Model.create(body);

            case 2:
              value = _context5.sent;

              if (value) {
                _context5.next = 5;
                break;
              }

              throw new errors.BreakingResponse({ message: 'Error occurred creating an item for "' + name + '" model.' });

            case 5:
              return _context5.abrupt('return', _defineProperty({}, singular(name), value));

            case 6:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, _this5);
    }));

    return function (_x10) {
      return _ref18.apply(this, arguments);
    };
  }();
};

/**
 * Update a resource item in the database.
 *
 * @param {string} name the resource name
 */
module.exports.update = function update() {
  var _this6 = this;

  var _ref20 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref20.name;

  expect({ name: 'name', value: name, type: 'string' });
  return function () {
    var _ref22 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(_ref21) {
      var params = _ref21.params,
          body = _ref21.body,
          Model = _ref21.Model;
      var id, value;
      return regeneratorRuntime.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              id = params[name + 'Id'];

              checkObjectId(id);
              _context6.next = 4;
              return Model.findById(id);

            case 4:
              value = _context6.sent;

              if (value) {
                _context6.next = 7;
                break;
              }

              throw new errors.NotFoundResponse({ message: 'Model ' + name + ' did not have an item with the id "' + id + '".' });

            case 7:
              _context6.next = 9;
              return merge(value, body).save();

            case 9:
              return _context6.abrupt('return', _defineProperty({}, singular(name), value));

            case 10:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, _this6);
    }));

    return function (_x12) {
      return _ref22.apply(this, arguments);
    };
  }();
};

/**
 * Remove a resource from the database.
 *
 * @param {string} name the resource name
 */
module.exports.remove = function remove() {
  var _this7 = this;

  var _ref24 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref24.name,
      safe = _ref24.safe,
      timestamps = _ref24.timestamps;

  expect({ name: 'name', value: name, type: 'string' });
  return function () {
    var _ref26 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(_ref25) {
      var params = _ref25.params,
          Model = _ref25.Model;
      var id, value, body;
      return regeneratorRuntime.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              id = params[name + 'Id'];

              checkObjectId(id);

              if (!safe) {
                _context7.next = 14;
                break;
              }

              _context7.next = 5;
              return Model.findById(id);

            case 5:
              value = _context7.sent;

              if (value) {
                _context7.next = 8;
                break;
              }

              throw new errors.NotFoundResponse({ message: 'Model ' + name + ' did not have an item with the id "' + id + '".' });

            case 8:
              body = { deleted: true };

              if (timestamps) body.deletedAt = new Date();
              _context7.next = 12;
              return merge(value, body).save();

            case 12:
              _context7.next = 16;
              break;

            case 14:
              _context7.next = 16;
              return Model.findByIdAndRemove(id);

            case 16:
              return _context7.abrupt('return', _defineProperty({}, singular(name), null));

            case 17:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, _this7);
    }));

    return function (_x14) {
      return _ref26.apply(this, arguments);
    };
  }();
};